/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
 *
 * Copyright (C) 2007-2012 Lucian Langa <cooly@gnome.eu.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *  02110-1301 USA
 */


#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib/gi18n-lib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/time.h>

#include <gtk/gtk.h>

#if (DATASERVER_VERSION >= 2031001)
#include <camel/camel.h>
#else
#include <camel/camel-mime-message.h>
#include <camel/camel-folder.h>
#include <camel/camel-exception.h>
#include <camel/camel-multipart.h>
#include <camel/camel-stream-mem.h>
#include <camel/camel-stream-fs.h>
#endif

#if EVOLUTION_VERSION >= 30706
#include <libedataserver/libedataserver.h>
#include <e-util/e-util.h>

#if EVOLUTION_VERSION >= 31000
#include <libemail-engine/libemail-engine.h>
#else
#include <libemail-engine/e-mail-folder-utils.h>
#include <libemail-engine/mail-ops.h>
#include <libemail-engine/mail-tools.h>
#include <shell/es-event.h>
#endif

#include <shell/e-shell-view.h>
#include <mail/e-mail-reader.h>
#include <mail/em-utils.h>
#include <mail/message-list.h>
#include <mail/em-folder-selector.h>

#else

#if EVOLUTION_VERSION < 22900
#include <mail/mail-component.h>
#include <mail/em-popup.h>
#include <misc/e-activity-handler.h>
#else
#if (EVOLUTION_VERSION < 30303)
#include <mail/e-mail-local.h>
#endif
#if EVOLUTION_VERSION >= 30305
#include <libemail-engine/e-mail-folder-utils.h>
#else
#if EVOLUTION_VERSION >= 30101
#include <mail/e-mail-folder-utils.h>
#endif
#endif
#include <mail/message-list.h>
#include <misc/e-popup-action.h>
#include <shell/e-shell-view.h>
#include <shell/es-event.h>
#include <mail/em-folder-selector.h>
#endif

#if EVOLUTION_VERSION <= 30501
#if EVOLUTION_VERSION >= 30305
#include <libemail-utils/e-account-utils.h>
#else
#include <e-util/e-account-utils.h>
#endif
#endif

#if (EVOLUTION_VERSION >= 30391)
#include <libevolution-utils/e-alert-dialog.h>
#else
#include <e-util/e-alert-dialog.h>
#endif
#include <e-util/e-mktemp.h>
#if EVOLUTION_VERSION >= 30305
#include <libemail-engine/mail-tools.h>
#include <libemail-engine/mail-config.h>
#else
#include <mail/mail-tools.h>
#include <mail/mail-config.h>
#endif
#include <mail/em-config.h>

#if EVOLUTION_VERSION <= 30501
#include <libedataserver/e-account-list.h>
#include <libedataserver/e-data-server-util.h>
#endif

#include <mail/em-utils.h>
#if EVOLUTION_VERSION >= 30304
#include <libemail-engine/mail-ops.h>
#else
#include <mail/mail-ops.h>
#endif
#endif /* EVOLUTION_VERSION >= 30706 */

#include "rspam.h"

#define d(x)

#if EVOLUTION_VERSION >= 30101
static EMailBackend *backend;
static EMailSession *session;
static EShellBackend *shell_backend;
#endif

#if EVOLUTION_VERSION >= 22900 //kb//
static EShellView *main_shell_view;
#endif


struct rspam_stuff {
	GtkBuilder  *xml;
#if EVOLUTION_VERSION < 30304
	EMConfigTargetPrefs *target;
#else
	GSettings *settings;
#endif
	GtkWidget *option_menu;
	GtkWidget *option_menu2;
	GtkWidget *option_menu3;
	GtkWidget *option_menu4;
	GtkWidget *username;
	GtkWidget *password;
	GtkWidget *home;
	GtkWidget *hbox;
	gchar *homepath;
	gchar *user;
	gchar *pass;
	gchar *spamcop_email;
};

struct _copy_folder_data {
#if EVOLUTION_VERSION >= 30101
	CamelStore *source_store;
	gchar *source_folder_name;
#else
	CamelFolderInfo *fi;
#endif
	gboolean delete;
};

void spamcop_report(CamelMimeMessage *msg, gchar *msgfile);
void taskbar_push_message(gchar *message);
void taskbar_pop_message(void);
void
#if (DATASERVER_VERSION >= 2031002)
rs_online(EShell *shell);
#else
rs_online(CamelObject *o, void *event_data, void *data);
#endif
void rspam_folder_cb (GtkWidget *widget, gpointer data);
gchar *rspam_lookup_uri_by_folder_name(gchar *name);

#define DISCOVERY_TTL		60*60*24*7
#define CNETS			4
#define SPAMCOP_LIMIT		50*1024
//temporary use spamassassin default address
//as spamcop guys did not respond
#define SPAMCOP_DEFAULT_EMAIL	"spamassassin-submit@spam.spamcop.net"

#define RAZORLINE RAZOREP

//colaborative network	spam command line		ham command line	gconf flag name
//		name
#if EVOLUTION_VERSION < 30304
static const char *cnets[CNETS][4] = {
	{"razor",	"razor-report -f -d",		"razor-revoke -f -d",	GCONF_KEY_ENABLE_RAZOR},
	{"dcc",	"dccproc -H -t many -x 0",	NULL,			GCONF_KEY_ENABLE_DCC},
	{"spamcop",(gchar *)spamcop_report,	NULL,			GCONF_KEY_ENABLE_SPAMCOP},
	{"pyzor",  "pyzor -d report",		"pyzor -d whitelist",	GCONF_KEY_ENABLE_PYZOR}};
#else
static const char *cnets[CNETS][4] = {
	{"razor",	"razor-report -f -d",		"razor-revoke -f -d",	CONF_KEY_ENABLE_RAZOR},
	{"dcc",	"dccproc -H -t many -x 0",	NULL,			CONF_KEY_ENABLE_DCC},
	{"spamcop",(gchar *)spamcop_report,	NULL,			CONF_KEY_ENABLE_SPAMCOP},
	{"pyzor",  "pyzor -d report",		"pyzor -d whitelist",	CONF_KEY_ENABLE_PYZOR}};
#endif

static gchar *pyzor_status = NULL;
#if EVOLUTION_VERSION < 30304
GConfClient *rspam_gconf;
#else
static GSettings *rspam_settings;
#endif
static struct rspam_stuff *stuff;
static gboolean rspam_online;
static GList *tlist = NULL;
static GPtrArray *spamuids = NULL;

static void
readrazorconfig(gpointer data);

//spam functions wrapper
//for calling internal functions
void
intern_call(void (*p)(), CamelMimeMessage *msg, gchar *msgfile)
{
	p(msg, msgfile);
}

/* Save email content as a file */
gchar*
save_message(CamelMimeMessage *msg)
{
	CamelStream *out;
	char *tmpdir = NULL, *name = NULL;
	tmpdir = e_mkdtemp("rspam-XXXXXX");
	if ( tmpdir == NULL)
		return (gchar *)-1;

	name = g_build_filename(tmpdir, ".evo-attachment.rspam", NULL);
	out = camel_stream_fs_new_with_name(name, O_RDWR|O_CREAT,
#if EVOLUTION_VERSION < 23191
			0666);
#else
			0666, NULL);
#endif

	if (!out)
		goto fail;

	// dump the whole message incl headers to report
#if (DATASERVER_VERSION >= 2033001)
	if (camel_data_wrapper_decode_to_stream_sync(
		(CamelDataWrapper *)msg, out, NULL, NULL) == -1
		|| camel_stream_close(out, NULL, NULL) == -1) {
#else
#if EVOLUTION_VERSION < 23191
	if (camel_data_wrapper_decode_to_stream(
		(CamelDataWrapper *)msg, out) == -1
		|| camel_stream_close(out) == -1) {
#else
	if (camel_data_wrapper_decode_to_stream(
		(CamelDataWrapper *)msg, out, NULL) == -1
		|| camel_stream_close(out, NULL) == -1) {
#endif
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(out);
#else
		camel_object_unref(out);
#endif
		goto fail;
	}
#if (DATASERVER_VERSION >= 2031001)
	g_object_unref(out);
#else
	camel_object_unref(out);
#endif
	return name;

fail:
	g_free(name);
	g_free(tmpdir);
	return NULL;
}

void
update_stats(gboolean command)
{
	FILE *STF;
	gchar buf[512];
	int spam=0;
	gchar *stfile;

	//update stats
	if (command)
		stfile = g_build_filename(
				(gchar *)e_get_user_data_dir(), ".rspam", NULL);
	else
		stfile = g_build_filename(
				(gchar *)e_get_user_data_dir(), ".rham", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		goto create;
	fgets(buf, 512, STF);
	fclose(STF);
	spam = atoi(buf);
create:	spam++;
	STF = fopen(stfile, "w+");
	if (!STF)
		return;
	fprintf(STF, "%d", spam);
	fclose(STF);
}

gint
rspam_command(CamelMimeMessage *msg, gchar *name, gint command) //command: 1 - spam; 0 - ham
{
	gint i;
	gchar *cmd, *cmdp;

	//later we have to check if we have a home and we're registered
	//remember to put -s when testing, in order no to clog cloudmark
	for (i=0; i < CNETS; i++) {
		g_print("CNET: %s\n", cnets[i][0]);
#if EVOLUTION_VERSION < 30304
		if (gconf_client_get_bool (rspam_gconf, cnets[i][3], NULL)) {
#else
		if (g_settings_get_boolean (rspam_settings, cnets[i][3])) {
#endif
			g_print("%s is enabled.\n", cnets[i][0]);
			if (command) {	/*spam*/
				if (!strncmp(cnets[i][0], "spamcop", 7))
					intern_call((void *)cnets[i][1], msg, name);
				else {
					if (rspam_online) {
#if EVOLUTION_VERSION < 30304
						if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL)) {
#else
						if (!g_settings_get_boolean(rspam_settings, CONF_KEY_DRY_RUN)) {
#endif
							cmdp = g_strconcat(cnets[i][1], " < %s &", NULL);
							cmd = g_strdup_printf(cmdp, name);
							system(cmd);
							g_free(cmd);
							g_free(cmdp);
							taskbar_push_message(_("Message reported as SPAM!"));
							update_stats(command);
						} else
						taskbar_push_message(_("Message reported as SPAM! (dry mode!)"));
					} else
						taskbar_push_message(_("Offline mode! Not reporting SPAM!"));

				}
			} else {		/*ham*/
				if (rspam_online) {
					if (NULL != cnets[i][2]) {
#if EVOLUTION_VERSION < 30304
						if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL)) {
#else
						if (!g_settings_get_boolean(rspam_settings, CONF_KEY_DRY_RUN)) {
#endif
							cmdp = g_strconcat(cnets[i][2], " < %s", NULL);
							cmd = g_strdup_printf(cmdp, name);
							system(cmd);
							g_free(cmd);
							g_free(cmdp);
						}
						taskbar_push_message(_("Message is no loger SPAM!"));
					} else {
						gchar *msg =
						g_strdup_printf(
							_("Revoke operation not supported on %s!"),
							cnets[i][0]);
						taskbar_push_message(msg);
						g_free(msg);
#ifdef DEBUG
						g_print("revoke not supported!\n");
#endif
					}
					update_stats(command);
				}
				else
					taskbar_push_message(_("Offline mode. Cannot revoke SPAM."));
			}
		} else
			g_print("%s is disabled. Skip test.\n", cnets[i][0]);
	}
	//let's just make sure we don't leave any junk on our way out,
	//even if we're in trash pile
	tlist = g_list_append(tlist, g_strdup(name));
	return 1;
}


void
spamcop_report(CamelMimeMessage *msg, gchar *msgfile)
{
	CamelStream *mem;
	int tzone;
	time_t date, now;
#if EVOLUTION_VERSION >= 30303
	EMailBackend *backend;
	EMailSession *session;
	EMailReader *reader;
	EShellContent *shell_content;
#endif
#if EVOLUTION_VERSION > 30501
	EShell *shell;
	ESource *source = NULL;
	ESourceRegistry *registry;
	ESourceMailIdentity *extension;
	const gchar *extension_name;
	const gchar *address, *name;
	const gchar *uid;
#else
	EAccount *account = NULL;
#endif
	gchar *descr;
	CamelFolder *mail_folder;
	CamelMessageInfo *info;
	struct _camel_header_raw *xev;
	CamelMimePart *part;
	CamelMimeMessage *new = camel_mime_message_new();
	gchar *text, *tmsg;
	CamelMultipart *body = camel_multipart_new();
	CamelInternetAddress *addr;
	gchar *sc_email;
	gssize len = 0;

	date = camel_mime_message_get_date_received(msg, &tzone);

	/*Convert to UTC*/
	date += (tzone / 100) * 60 * 60;
	date += (tzone % 100) * 60;
	time(&now);

	if (date && date < now - 2*86400) {
		taskbar_push_message(
			_("SpamCop: Message too old, not reporting mail older than 2 days."));
#ifdef DEBUG
		g_print("SpamCop: Message too old, not reporting mail older than 2 days.\n");
#endif
		d(g_print("now:%d\n", (guint)now - 2*86400));
		d(g_print("date:%d\n", (guint)date));
		goto fail;
	}

#if EVOLUTION_VERSION >= 30303
	shell_content = e_shell_view_get_shell_content (main_shell_view);
	reader = E_MAIL_READER (shell_content);
	backend = e_mail_reader_get_backend (reader);
	session = e_mail_backend_get_session (backend);
#endif

#if EVOLUTION_VERSION < 30303
#if EVOLUTION_VERSION < 22900
	if (!em_utils_check_user_can_send_mail(NULL)) {
		e_error_run(NULL,
			"mail:send-no-account-enabled",
			NULL);
#else
	if (!em_utils_check_user_can_send_mail()) {
		e_alert_run_dialog_for_args(
			e_shell_get_active_window (NULL),
			"mail:send-no-account-enabled",
			NULL);
#endif
		goto fail;
	}
#endif

#if EVOLUTION_VERSION > 30501
	shell = e_shell_get_default ();
	registry = e_shell_get_registry (shell);
	source = e_source_registry_ref_default_mail_identity (registry);
	extension_name = E_SOURCE_EXTENSION_MAIL_IDENTITY;
	extension = e_source_get_extension (source, extension_name);
	name = e_source_mail_identity_get_name (extension);
	address = e_source_mail_identity_get_address (extension);
	uid = e_source_get_uid (source);

	if (name == NULL && address == NULL)
		goto fail;
#else
#if EVOLUTION_VERSION < 22900
	account = mail_config_get_default_account ();
#else
	account = e_get_default_account ();
#endif
	if ( !account || (account->transport == NULL)
		|| (account->transport->url == NULL)
		|| (*account->transport->url == '\0'))
		goto fail;
#endif

	// check and trim if message is above spamcop limit 50k
	// couldn't find better method at this time :-)
#if EVOLUTION_VERSION < 22304
	text = em_utils_message_to_html (msg, NULL, 0, &len, NULL);
#else
#if EVOLUTION_VERSION < 22900
	text = em_utils_message_to_html (msg, NULL, 0, &len, NULL, NULL);
#else
#if EVOLUTION_VERSION < 29190
	text = em_utils_message_to_html (msg, NULL, 0, &len, NULL, NULL, NULL);
#else
#if EVOLUTION_VERSION > 30400
#if EVOLUTION_VERSION < 31303
	text = em_utils_message_to_html (CAMEL_SESSION (session), msg, NULL, 0, NULL, NULL, NULL);
#else
	text = em_utils_message_to_html (CAMEL_SESSION (session), msg, NULL, 0, NULL, NULL, NULL, NULL);
#endif
#else
	text = em_utils_message_to_html (msg, NULL, 0, NULL, NULL, NULL);
#endif
#endif
#endif
#endif
	if ( SPAMCOP_LIMIT < len) {
		gchar *tmp;
		tmsg = g_strndup(text,
			SPAMCOP_LIMIT);
		tmp = g_strconcat(
				(gchar *)tmsg,
				"[message truncated by evolution RSPAM]",
				NULL);
		g_free(tmsg);
		mem = camel_stream_mem_new();
#if (DATASERVER_VERSION >= 2033001)
		camel_stream_write (mem, tmp, strlen(tmp), NULL, NULL);
#else
#if EVOLUTION_VERSION < 23191
		camel_stream_write(mem, tmp, strlen(tmp));
		camel_stream_reset(mem);
#else
		camel_stream_write(mem, tmp, strlen(tmp), NULL);
		camel_stream_reset(mem, NULL);
#endif
#endif
		g_free(tmp);
		msg = camel_mime_message_new();
#if (DATASERVER_VERSION >= 2033001)
		camel_data_wrapper_construct_from_stream_sync (
			CAMEL_DATA_WRAPPER(msg), mem, NULL, NULL);
#else
#if EVOLUTION_VERSION < 23191
		camel_data_wrapper_construct_from_stream (
			CAMEL_DATA_WRAPPER(msg), mem);
#else
		camel_data_wrapper_construct_from_stream (
			CAMEL_DATA_WRAPPER(msg), mem, NULL);
#endif
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(mem);
#else
		camel_object_unref(mem);
#endif
	}
	g_free(text);

#if EVOLUTION_VERSION < 22900
		mail_folder = mail_component_get_folder(
				NULL,
				MAIL_COMPONENT_FOLDER_OUTBOX);
#else
#if EVOLUTION_VERSION < 29102
		mail_folder = e_mail_local_get_folder(E_MAIL_FOLDER_OUTBOX);
#else
#if EVOLUTION_VERSION < 30303
		mail_folder = e_mail_local_get_folder(E_MAIL_LOCAL_FOLDER_OUTBOX);
#else
		mail_folder = e_mail_session_get_local_folder (
				session, E_MAIL_LOCAL_FOLDER_OUTBOX);
#endif
#endif

#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_ref(mail_folder);
#else
		camel_object_ref(mail_folder);
#endif

		info = camel_message_info_new(NULL);
		camel_message_info_set_flags(
			info,
			CAMEL_MESSAGE_SEEN,
			~0);

		camel_mime_message_set_subject(
			new,
			"report spam");

		addr = camel_internet_address_new();
#if EVOLUTION_VERSION < 30304
		sc_email = gconf_client_get_string(
				rspam_gconf,
				GCONF_KEY_SPAMCOP_EMAIL,
				NULL);
#else
		sc_email = g_settings_get_string(
				rspam_settings,
				CONF_KEY_SPAMCOP_EMAIL);
#endif
		camel_address_decode(
			CAMEL_ADDRESS(addr),
			sc_email ? sc_email : SPAMCOP_DEFAULT_EMAIL);
		camel_mime_message_set_recipients(
			new,
			CAMEL_RECIPIENT_TYPE_TO,
			addr);
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(addr);
#else
		camel_object_unref(addr);
#endif

		addr = camel_internet_address_new();
		d(g_print("name:%s\n", account->id->name));
		d(g_print("address:%s\n", account->id->address));
		camel_internet_address_add(
			addr,
#if EVOLUTION_VERSION > 30501
			name,
			address);
#else
			account->id->name,
			account->id->address);
#endif
		camel_mime_message_set_from(
			new,
			addr);
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(addr);
#else
		camel_object_unref(addr);
#endif

		xev = mail_tool_remove_xevolution_headers (msg);
		camel_header_raw_clear(&xev);

		camel_medium_remove_header (
			CAMEL_MEDIUM (msg),
			"Bcc");

	part = camel_mime_part_new ();
#if (DATASERVER_VERSION >= 2031001)
	camel_medium_set_content (
		CAMEL_MEDIUM (part),
		CAMEL_DATA_WRAPPER (msg));
#else
	camel_medium_set_content_object (
		CAMEL_MEDIUM (part),
		CAMEL_DATA_WRAPPER (msg));
#endif

	camel_mime_part_set_content_type(
		part,
		"message/rfc822;"
		" x-spam-type=report");
	descr = g_strdup_printf(
		"spam report via RSPAM-%s",
		VERSION);
	camel_mime_part_set_description(
		part,
		descr);
	camel_mime_part_set_disposition(
		part,
		"attachment");
	camel_medium_set_header(
		CAMEL_MEDIUM(part),
		"Content-Transfer-Encoding",
		"8bit");
//    camel_medium_set_header(part, "X-Spam-Relays-Trusted", "");
//    camel_medium_set_header(part, "X-Spam-Relays-Untrusted", "");

	camel_data_wrapper_set_mime_type(
		CAMEL_DATA_WRAPPER(body),
		"multipart/mixed;"
		"boundary=\"\"");
	camel_multipart_set_boundary(
		body,
		NULL);
	camel_multipart_add_part(
		body,
		part);
#if (DATASERVER_VERSION >= 2031001)
	g_object_unref(part);
#else
	camel_object_unref(part);
#endif

#if EVOLUTION_VERSION > 30501
	if (source != NULL) {
		ESourceMailSubmission *ms;
		const gchar *header_name;
		const gchar *transport_uid;
		const gchar *sent_folder;

		extension_name = E_SOURCE_EXTENSION_MAIL_SUBMISSION;
		ms = e_source_get_extension (source, extension_name);

		sent_folder = e_source_mail_submission_get_sent_folder (ms);
		transport_uid = e_source_mail_submission_get_transport_uid (ms);

		/* X-Evolution-Account */
		header_name = "X-Evolution-Account";
		camel_medium_set_header (CAMEL_MEDIUM(new), header_name, uid);

		/* X-Evolution-Fcc */
		header_name = "X-Evolution-Fcc";
		camel_medium_set_header (CAMEL_MEDIUM(new), header_name, sent_folder);

		/* X-Evolution-Transport */
		header_name = "X-Evolution-Transport";
		camel_medium_set_header (CAMEL_MEDIUM(new), header_name, transport_uid);

		g_object_unref (source);
	}
#endif

	/*more voodoo here*/

#if EVOLUTION_VERSION < 30304
	if (!gconf_client_get_bool(rspam_gconf, GCONF_KEY_DRY_RUN, NULL)) {
#else
	if (!g_settings_get_boolean(rspam_settings, CONF_KEY_DRY_RUN)) {
#endif
#if (DATASERVER_VERSION >= 2031001)
		camel_medium_set_content(
			CAMEL_MEDIUM(new),
			CAMEL_DATA_WRAPPER(body));
#else
		camel_medium_set_content_object(
			CAMEL_MEDIUM(new),
			CAMEL_DATA_WRAPPER(body));
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref(body);
#else
		camel_object_unref(body);
#endif

#if EVOLUTION_VERSION < 29102
		mail_append_mail(mail_folder,
			new, info, NULL, NULL);
#else
		e_mail_folder_append_message(
			mail_folder, new, info,
			G_PRIORITY_DEFAULT, NULL,
			NULL, NULL);
#endif
		/*composer_send_queued_cb*/

		update_stats(1);
	}
fail:	return;
}

guint
read_spam(void)
{
	gchar *stfile;
	FILE *STF;
	gchar buf[512];

	stfile = g_build_filename((gchar *)e_get_user_data_dir(), ".rspam", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		return 0;
	fgets(buf, 512, STF);
	fclose(STF);
	return atoi(buf);
}

guint
read_ham(void)
{
	gchar *stfile;
	FILE *STF;
	gchar buf[512];

	stfile = g_build_filename((gchar *)e_get_user_data_dir(), ".rham", NULL);
	STF = fopen(stfile, "r");
	if (!STF)
		return 0;
	fgets(buf, 512, STF);
	fclose(STF);
	return atoi(buf);
}

#if EVOLUTION_VERSION >= 30101
static void
rss_closure_free (RSSEMailReaderClosure *closure)
{
	if (closure->reader != NULL)
		g_object_unref (closure->reader);

	if (closure->activity != NULL)
		g_object_unref (closure->activity);

	g_slice_free (RSSEMailReaderClosure, closure);
}
#endif

void rspam_action_do (CamelFolder *folder,
#if EVOLUTION_VERSION >= 30101
			GAsyncResult *result,
			RSSEMailReaderClosure *closure)
#else
			const char *uid,
			CamelMimeMessage *msg,
			void *data)
#endif
{
	gchar *name;
#if EVOLUTION_VERSION >= 30101
	EAlertSink *alert_sink;
	CamelMimeMessage *msg;
	GError *error = NULL;

	alert_sink = e_activity_get_alert_sink (closure->activity);

	msg = camel_folder_get_message_finish (folder, result, &error);

	if (e_activity_handle_cancellation (closure->activity, error)) {
		g_warn_if_fail (msg == NULL);
		rss_closure_free (closure);
		g_error_free (error);
		return;
	} else if (error != NULL) {
		g_warn_if_fail (msg == NULL);
		e_alert_submit (
			alert_sink, "mail:no-retrieve-message",
			error->message, NULL);
		rss_closure_free (closure);
		g_error_free (error);
		return;
	}

	g_warn_if_fail (CAMEL_IS_MIME_MESSAGE (msg));
	if (!CAMEL_IS_MIME_MESSAGE (msg)) {
		rss_closure_free (closure);
		return;
	}
#endif

	name = save_message(msg);
	if (name) {
		rspam_command(msg, name, 1);
		g_free(name);
	}

#if EVOLUTION_VERSION >= 30101
	rss_closure_free (closure);
	g_object_unref (msg);
#endif
}

void
mark_junk(gchar *uid, CamelFolder *folder)
{
	camel_folder_set_message_flags(folder, uid,
		CAMEL_MESSAGE_JUNK,
		CAMEL_MESSAGE_JUNK);
}

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_rspam(void *ep, EMPopupTargetSelect *t);
#else
org_gnome_sa_rspam (GtkAction *action,
		EShellView *shell_view);
#endif

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_rspam(void *ep, EMPopupTargetSelect *t)
#else
org_gnome_sa_rspam(GtkAction *action, EShellView *shell_view)
#endif
{
	CamelFolder *folder;
	GPtrArray *uids;
	gint i;
#if EVOLUTION_VERSION < 22900
	uids = t->uids;
	folder = t->folder;
#else
	EShellContent *shell_content;
	EMailReader *reader;
#if EVOLUTION_VERSION >= 30101
	EActivity *activity;
	GCancellable *cancellable;
	RSSEMailReaderClosure *closure;
#endif

	shell_content = e_shell_view_get_shell_content (shell_view);

	reader = E_MAIL_READER (shell_content);
#if EVOLUTION_VERSION >= 31190
	folder = e_mail_reader_ref_folder(reader);
#else
	folder = e_mail_reader_get_folder(reader);
#endif
	uids = e_mail_reader_get_selected_uids(reader);
#endif

	while (gtk_events_pending())
		gtk_main_iteration ();

	if (spamuids)
		g_ptr_array_free(spamuids, FALSE);

	spamuids = g_ptr_array_new();

	for (i = 0; i < (uids ? uids->len : 0); i++) {
		d(g_print("i:%d, uids:%s\n", i, uids->pdata[i]));
#if EVOLUTION_VERSION < 30101
		mail_get_message (
			folder,
			uids->pdata[i],
			rspam_action_do, NULL, mail_msg_unordered_push);
#else
		activity = e_mail_reader_new_activity (reader);
		cancellable = e_activity_get_cancellable (activity);
		closure = g_slice_new0 (RSSEMailReaderClosure);
		closure->activity = activity;
		closure->reader = g_object_ref (reader);
		camel_folder_get_message(
			folder,
			uids->pdata[i],
			G_PRIORITY_DEFAULT,
			cancellable, (GAsyncReadyCallback)
			rspam_action_do, closure);
#endif

		g_ptr_array_add(spamuids, uids->pdata[i]);
	}

#if EVOLUTION_VERSION < 30304
	if (!gconf_client_get_bool (rspam_gconf, GCONF_KEY_MOVE_JUNK, NULL)) {
#else
	if (!g_settings_get_boolean (rspam_settings, CONF_KEY_MOVE_JUNK)) {
#endif
		gchar *uri;
#if EVOLUTION_VERSION >= 30303
		EMailBackend *backend;
		EMailSession *session;
		EMailReader *reader;
		EShellContent *shell_content;

		shell_content = e_shell_view_get_shell_content (main_shell_view);
		reader = E_MAIL_READER (shell_content);
		backend = e_mail_reader_get_backend (reader);
		session = e_mail_backend_get_session (backend);
#endif
#if EVOLUTION_VERSION < 30304
		uri = gconf_client_get_string (rspam_gconf, GCONF_KEY_MOVE_FOLDER, NULL);
#else
		uri = g_settings_get_string (rspam_settings, CONF_KEY_MOVE_FOLDER);
#endif
		if (uri && g_ascii_strcasecmp(uri, _("Select...")) && spamuids->len) {
			mail_transfer_messages (
#if EVOLUTION_VERSION < 29101
				folder, spamuids,
#else
				session, folder, spamuids,
#endif
				TRUE, uri, 0, NULL, NULL);
			g_free(uri);
		}
	} else
		g_ptr_array_foreach(spamuids, (GFunc)mark_junk, folder);

	while (gtk_events_pending())
		gtk_main_iteration ();

#if EVOLUTION_VERSION >= 31190
	g_clear_object (&folder);
#endif
}

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_revoke(void *ep, EMPopupTargetSelect *t);
#else
org_gnome_sa_revoke (GtkAction *action,
		EShellView *shell_view);
#endif

void
#if EVOLUTION_VERSION < 22900
org_gnome_sa_revoke(void *ep, EMPopupTargetSelect *t)
#else
org_gnome_sa_revoke (GtkAction *action,
		EShellView *shell_view)
#endif
{
	CamelFolder *folder;
	GPtrArray *uids;
	CamelMimeMessage *msg;
	gchar *name;
	gint i;
#if EVOLUTION_VERSION < 22900
	uids = t->uids;
	folder = t->folder;
#else
	EShellContent *shell_content;
	EMailReader *reader;
	GtkWidget *message_list;
	shell_content = e_shell_view_get_shell_content (shell_view);

	reader = E_MAIL_READER (shell_content);
	message_list = e_mail_reader_get_message_list (reader);
#if EVOLUTION_VERSION >= 31190
	folder = message_list_ref_folder (MESSAGE_LIST(message_list));
#else
	folder = MESSAGE_LIST(message_list)->folder;
#endif
	uids = message_list_get_selected (MESSAGE_LIST(message_list));
#endif

	while (gtk_events_pending())
		gtk_main_iteration ();

	for (i = 0; i < (uids ? uids->len : 0); i++) {
#if (DATASERVER_VERSION >= 2033001)
		msg = camel_folder_get_message_sync (folder,
			g_ptr_array_index(uids, i), NULL, NULL);
#else
		msg = camel_folder_get_message (folder,
			g_ptr_array_index(uids, i), NULL);
#endif
		if (!msg)
			continue;

	name = save_message(msg);
	if (name) {
		rspam_command(msg, name, 0);
		g_free(name);
	}

#if EVOLUTION_VERSION < 30304
	if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_MOVE_JUNK, NULL))
#else
	if (g_settings_get_boolean (rspam_settings, CONF_KEY_MOVE_JUNK))
#endif
		camel_folder_set_message_flags(
			folder, g_ptr_array_index(uids, i),
			CAMEL_MESSAGE_SEEN|CAMEL_MESSAGE_JUNK,
			CAMEL_MESSAGE_SEEN);

#if (DATASERVER_VERSION >= 2031001)
	g_object_unref(msg);
#else
	camel_object_unref(msg);
#endif
	}

	while (gtk_events_pending())
		gtk_main_iteration ();

#if EVOLUTION_VERSION >= 31190
	g_clear_object (&folder);
#endif
}


gchar*
pyzor_discover_cb (GtkWidget *widget, gpointer data)
{
	GByteArray *output_buffer = g_byte_array_new ();
	int out_fds[2];
	CamelStream *stream;
	pid_t pid;
	int i=0;
	char *argv[30];
	gchar *tmp;

	if (!rspam_online)
		return 0;

	argv[i++] = (char *)"pyzor";
	argv[i++] = (char *)"discover";
	argv[i] = NULL;

	if (output_buffer && pipe (out_fds) == -1) {
		return 0;
	}

	if (!(pid = fork ())) {
		/* child process */
		int maxfd, fd;

		if (dup2 (out_fds[1], STDOUT_FILENO) == -1)
			_exit (1 & 0377);
		if (output_buffer)
			close (out_fds [1]);

		setsid ();

		maxfd = sysconf (_SC_OPEN_MAX);
		for (fd = 3; fd < maxfd; fd++)
			fcntl (fd, F_SETFD, FD_CLOEXEC);

		execvp (argv[0], argv);
		_exit (1 & 0377);
	} else if (pid < 0) {
		return 0;
	}

	if (output_buffer)
		close (out_fds [1]);

	if (output_buffer) {
		CamelStreamMem *memstream;

		stream = camel_stream_fs_new_with_fd (out_fds[0]);

		memstream = (CamelStreamMem *) camel_stream_mem_new ();
		camel_stream_mem_set_byte_array (memstream, output_buffer);

#if (DATASERVER_VERSION >= 2033001)
		camel_stream_write_to_stream (stream,
			(CamelStream *) memstream, NULL, NULL);
#else
#if EVOLUTION_VERSION < 23191
		camel_stream_write_to_stream (stream,
			(CamelStream *) memstream);
#else
		camel_stream_write_to_stream (stream,
			(CamelStream *) memstream, NULL);
#endif
#endif
#if (DATASERVER_VERSION >= 2031001)
		g_object_unref (stream);
#else
		camel_object_unref (stream);
#endif
		g_byte_array_append (output_buffer, (guint8 *)"", 1);

		printf (
			"child process output: %s len: %d\n",
			output_buffer->data, output_buffer->len);
	}
	tmp = g_strdup((gchar *)output_buffer->data);

	return tmp;
}

static void
discover_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	const gchar *home = stuff->homepath;
	const gchar *user = stuff->user;
	const gchar *pass = stuff->pass;
	gchar *cmd;

	if (!rspam_online)
		return;

	if (home && user && pass)
		cmd = g_strdup_printf(
			"razor-admin -home %s -discover -user %s -pass %s &",
			home, user, pass);
	else
		cmd = g_strdup("razor-admin -discover &");
#ifdef DEBUG
	g_print("Razor discover - cmd:%s\n", cmd);
#endif
	system(cmd);
	g_free(cmd);
}

static void
register_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	const gchar *home = gtk_entry_get_text(GTK_ENTRY(stuff->home));
	const gchar *user = gtk_entry_get_text(GTK_ENTRY(stuff->username));
	const gchar *pass = gtk_entry_get_text(GTK_ENTRY(stuff->password));
	gchar *cmd;

	if (!rspam_online)
		return;

	cmd = g_strdup_printf(
		"razor-admin -d -home %s -register -user %s -pass %s",
		home, user, pass);
#ifdef DEBUG
	g_print("Razor register - cmd:%s\n", cmd);
#endif
	system(cmd);
	g_free(cmd);
	readrazorconfig(stuff);
}

static void
razor_home_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	const gchar *razor_home = gtk_entry_get_text(GTK_ENTRY(widget));
#if EVOLUTION_VERSION < 30304
	stuff->target->gconf = gconf_client_get_default();
#else
	stuff->settings = g_settings_new(CONF_SCHEMA);
#endif

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_string (
		stuff->target->gconf,
		GCONF_KEY_RAZOR_HOME, razor_home, NULL);
#else
	g_settings_set_string (
		stuff->settings,
		CONF_KEY_RAZOR_HOME, razor_home);
#endif
}

static void
spamcop_email_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	const gchar *spamcop_email = gtk_entry_get_text(GTK_ENTRY(widget));

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_string (
		stuff->target->gconf,
		GCONF_KEY_SPAMCOP_EMAIL, spamcop_email, NULL);
#else
	g_settings_set_string (
		stuff->settings,
		CONF_KEY_SPAMCOP_EMAIL, spamcop_email);
#endif
}

static void
enable_dcc_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_ENABLE_DCC, active, NULL);
#else
	g_settings_set_boolean (
		stuff->settings, CONF_KEY_ENABLE_DCC, active);
#endif
}

static void
enable_razor_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_bool (
		stuff->target->gconf,
		GCONF_KEY_ENABLE_RAZOR, active, NULL);
#else
	g_settings_set_boolean (
		stuff->settings,
		CONF_KEY_ENABLE_RAZOR, active);
#endif
}

static void
enable_spamcop_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_bool (
		stuff->target->gconf,
		GCONF_KEY_ENABLE_SPAMCOP, active, NULL);
#else
	g_settings_set_boolean (
		stuff->settings,
		CONF_KEY_ENABLE_SPAMCOP, active);
#endif
}

static void
enable_movejunk_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	gtk_widget_set_sensitive(stuff->hbox, 1-active);

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_MOVE_JUNK, active, NULL);
#else
	g_settings_set_boolean(rspam_settings, CONF_KEY_MOVE_JUNK, active);
#endif
}

static void
enable_dryrun_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_DRY_RUN, active, NULL);
#else
	g_settings_set_boolean (
		stuff->settings, CONF_KEY_DRY_RUN, active);
#endif
}

static void
enable_pyzor_cb (GtkWidget *widget, gpointer data)
{
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
	gboolean active = gtk_toggle_button_get_active (
				GTK_TOGGLE_BUTTON (widget));

	/* Save the new setting */
#if EVOLUTION_VERSION < 30304
	gconf_client_set_bool (
		stuff->target->gconf, GCONF_KEY_ENABLE_PYZOR, active, NULL);
#else
	g_settings_set_boolean (
		stuff->settings, CONF_KEY_ENABLE_PYZOR, active);
#endif
}

static void
readrazorconfig(gpointer data)
{
	gchar buf[512];
	char **str2;
	FILE *file;
	gchar *key, *path;
	gchar *str, *cmd;
	gchar *tmp;
	struct rspam_stuff *stuff = (struct rspam_stuff *) data;
#if EVOLUTION_VERSION < 30304
	rspam_gconf = gconf_client_get_default();
#else
	rspam_settings = g_settings_new(CONF_SCHEMA);
#endif

	//we might had razor before, so try to fetch it's home
	//otherwise we'll build ovious razor home from homedir
#if EVOLUTION_VERSION < 30304
	key = gconf_client_get_string(
		rspam_gconf, GCONF_KEY_RAZOR_HOME, NULL);
#else
	key = g_settings_get_string(
		rspam_settings, CONF_KEY_RAZOR_HOME);
#endif
	if (!key || !strlen(key)) {
		str = g_build_filename(
			g_get_home_dir(), ".razor", "identity", NULL);
		tmp = g_path_get_dirname(str);
#if EVOLUTION_VERSION < 30304
		gconf_client_set_string (
			rspam_gconf, GCONF_KEY_RAZOR_HOME, tmp, NULL);
#else
		g_settings_set_string (
			rspam_settings, CONF_KEY_RAZOR_HOME, tmp);
#endif
		g_free(tmp);
#if EVOLUTION_VERSION < 30304
		key = gconf_client_get_string(
			rspam_gconf, GCONF_KEY_RAZOR_HOME, NULL);
#else
		key = g_settings_get_string(
			rspam_settings, CONF_KEY_RAZOR_HOME);
#endif
	} else {
		//we  have a razor home stored
		str = g_build_filename(key, "identity", NULL);
	}
	stuff->homepath = key;

	//try to see if files really exists
	file = fopen(str, "rb");
	if (!file) {
		//identity file does not exist, let's try home
		path = g_path_get_dirname(str);
		if (!g_file_test(path, G_FILE_TEST_IS_DIR)) {
			//create
			cmd = g_strdup_printf(
				"razor-admin -d -home %s -create",
				path);
		g_print("%s\n", cmd);
		system(cmd);
		g_free(cmd);
		}
		//register
		cmd = g_strdup_printf("razor-admin -d -home %s -register",
				path);
		g_print("%s\n", cmd);
		system(cmd);
		g_free(cmd);
		//file must now be created
		//to we should check status
		file = fopen(str, "rb");
		if (!file) {
			stuff->user = NULL;
			stuff->pass = NULL;
			return;
		}
	}

	while(fgets(buf, 512, file)) {
		g_strstrip(buf);
		str2 = g_strsplit(buf, "=", 0);
		g_strstrip(str2[1]);
		if (!strncmp(str2[0], "user", 4))
			stuff->user = str2[1];
		if (!strncmp(str2[0], "pass", 4))
			stuff->pass = str2[1];
	}
	fclose(file);
}

void
taskbar_push_message(gchar *message)
{
#if 0
	EActivityHandler *activity_handler =
			mail_component_peek_activity_handler (
				mail_component_peek ());
	e_activity_handler_set_message(activity_handler, message);
#endif
}

void
taskbar_pop_message(void)
{
#if EVOLUTION_VERSION < 22900 //kb//
	EActivityHandler *activity_handler =
			mail_component_peek_activity_handler (
						mail_component_peek ());
	e_activity_handler_unset_message(activity_handler);
#else
	EShellTaskbar *shell_taskbar;
	g_return_if_fail(main_shell_view != NULL);
	shell_taskbar = e_shell_view_get_shell_taskbar (main_shell_view);
	e_shell_taskbar_set_message (shell_taskbar, "");
#endif

}

//=================
//== guification ==
//=================

GtkWidget *
rspam_page_factory (EPlugin *ep, EConfigHookItemFactoryData *hook_data)
{
	struct rspam_stuff *stuff;
#if EVOLUTION_VERSION < 30304
	EMConfigTargetPrefs *target = (EMConfigTargetPrefs *) hook_data->config->target;
#else
	GSettings *settings = g_settings_new (CONF_SCHEMA);
#endif
	GtkWidget *page;
	GtkWidget *tab_label;
	GtkWidget *check;
	GtkWidget *button;
	GtkWidget *entry1,*entry2,*entry3;
	GtkWidget *label1;
	gchar     *fld;
	gchar     *folder = NULL;
	gboolean  mj;
	gchar *sc_email;
	gchar *tmp, *uifile;
	GError* error = NULL;
	gchar *toplevel[] = {(gchar *)"settingsbox", NULL};
	guint spam;

	/* A structure to pass some stuff around */
	stuff = g_new0 (struct rspam_stuff, 100);
#if EVOLUTION_VERSION < 30304
	stuff->target = target;
#else
	stuff->settings = settings;
#endif

	readrazorconfig(stuff);


	uifile = g_build_filename (EVOLUTION_UIDIR,
			"rspam.ui",
			NULL);
	stuff->xml = gtk_builder_new ();
#if GTK_CHECK_VERSION (2,14,0)
	if (!gtk_builder_add_objects_from_file (stuff->xml, uifile, toplevel, &error)) {
		g_warning ("Couldn't load builder file: %s", error->message);
		g_error_free (error);
	}
#else
	g_warning("gtk too old! cannot create ui file. need >= 2.14\n");
	g_free(stuff);
	// and not very interesed to back port this
	return NULL;
#endif
	g_free (uifile);

	check = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "checkbutton1"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
#if EVOLUTION_VERSION < 30304
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_RAZOR, NULL));
#else
		g_settings_get_boolean (settings, CONF_KEY_ENABLE_RAZOR));
#endif
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_razor_cb), stuff);

	check = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "checkbutton2"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
#if EVOLUTION_VERSION < 30304
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_PYZOR, NULL));
#else
		g_settings_get_boolean (settings, CONF_KEY_ENABLE_PYZOR));
#endif
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_pyzor_cb), stuff);

	check = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "checkbutton3"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
#if EVOLUTION_VERSION < 30304
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_DCC, NULL));
#else
		g_settings_get_boolean (settings, CONF_KEY_ENABLE_DCC));
#endif
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_dcc_cb), stuff);

	check = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "checkbutton4"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
#if EVOLUTION_VERSION < 30304
		gconf_client_get_bool (target->gconf, GCONF_KEY_ENABLE_SPAMCOP, NULL));
#else
		g_settings_get_boolean (settings, CONF_KEY_ENABLE_SPAMCOP));
#endif
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_spamcop_cb), stuff);

	stuff->hbox = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "hbox11"));
	check = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "checkbutton5"));
#if EVOLUTION_VERSION < 30304
	mj = gconf_client_get_bool (target->gconf, GCONF_KEY_MOVE_JUNK, NULL);
#else
	mj = g_settings_get_boolean (settings, CONF_KEY_MOVE_JUNK);
#endif
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
		mj);
	gtk_widget_set_sensitive(stuff->hbox,
		1 - mj);
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_movejunk_cb), stuff);


	check = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "checkbutton6"));
	gtk_toggle_button_set_active (
		GTK_TOGGLE_BUTTON (check),
#if EVOLUTION_VERSION < 30304
		gconf_client_get_bool (target->gconf, GCONF_KEY_DRY_RUN, NULL));
#else
		g_settings_get_boolean (settings, CONF_KEY_DRY_RUN));
#endif
	g_signal_connect (
		GTK_TOGGLE_BUTTON (check),
		"toggled", G_CALLBACK (enable_dryrun_cb), stuff);

	entry1 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "entry1"));
	gtk_entry_set_invisible_char (GTK_ENTRY (entry1), 8226);
	gtk_entry_set_text(
		GTK_ENTRY(entry1),
#if EVOLUTION_VERSION < 30304
		gconf_client_get_string(target->gconf, GCONF_KEY_RAZOR_HOME, NULL));
#else
		g_settings_get_string(settings, CONF_KEY_RAZOR_HOME));
#endif
	g_signal_connect (
		GTK_ENTRY (entry1), "changed",
		G_CALLBACK (razor_home_cb), stuff);
	stuff->home = entry1;

	entry2 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "entry2"));
	gtk_entry_set_text(GTK_ENTRY(entry2), (gpointer)stuff->user);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry2), 8226);
	stuff->username = entry2;

	entry3 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "entry3"));
	gtk_entry_set_visibility (GTK_ENTRY (entry3), FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry3), (gpointer)stuff->pass);
	gtk_entry_set_invisible_char (GTK_ENTRY (entry3), 8226);
	stuff->password = entry3;

	button = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "button1"));
	g_signal_connect (
		GTK_BUTTON (button),
		"clicked", G_CALLBACK (register_cb), stuff);

	button = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "button2"));
	g_signal_connect (
		GTK_BUTTON (button), "clicked",
		G_CALLBACK (discover_cb), stuff);

	button = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "button3"));
	label1 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "label16"));
#if EVOLUTION_VERSION < 30304
	fld = gconf_client_get_string(
				target->gconf,
				GCONF_KEY_MOVE_FOLDER,
				NULL);
#else
	fld = g_settings_get_string(
				settings,
				CONF_KEY_MOVE_FOLDER);
#endif
	if (fld && g_ascii_strcasecmp(fld, _("Select..."))) {
#if EVOLUTION_VERSION >= 30101
		EMailBackend *backend;
		EMailReader *reader;
		EShellContent *shell_content;
		CamelStore *store;
		EMailSession *session;

		shell_content = e_shell_view_get_shell_content (main_shell_view);
		reader = E_MAIL_READER (shell_content);
		backend = e_mail_reader_get_backend (reader);
		session = e_mail_backend_get_session (backend);
		e_mail_folder_uri_parse (
			CAMEL_SESSION(session), fld,
			&store, &folder, NULL);
#else
		folder = em_utils_folder_name_from_uri(fld);
#endif
		g_free(fld);
		gtk_label_set_text(GTK_LABEL(label1), folder);
		g_free(folder);
	} else
		gtk_label_set_text(GTK_LABEL(label1), _("Select..."));
	g_signal_connect (
		GTK_BUTTON (button),
		"clicked", G_CALLBACK (rspam_folder_cb), label1);

	entry1 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "entry4"));
	gtk_entry_set_invisible_char (GTK_ENTRY (entry1), 8226);
#if EVOLUTION_VERSION < 30304
	sc_email = gconf_client_get_string(
				target->gconf,
				GCONF_KEY_SPAMCOP_EMAIL,
				NULL);
#else
	sc_email = g_settings_get_string(
				settings,
				CONF_KEY_SPAMCOP_EMAIL);
#endif
	stuff->spamcop_email = (sc_email && strlen(sc_email)) ? sc_email : (gchar *)SPAMCOP_DEFAULT_EMAIL;
	gtk_entry_set_text(GTK_ENTRY(entry1),
		stuff->spamcop_email);
	g_signal_connect (
		GTK_ENTRY (entry1),
		"changed", G_CALLBACK (spamcop_email_cb), stuff);

	spam = read_spam();
	tmp = g_strdup_printf(_("Spam reported: %d messages"), spam?spam:0);
	label1 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "label22"));
	gtk_label_set_text_with_mnemonic(GTK_LABEL(label1), tmp);
	g_free(tmp);

	spam = read_ham();
	tmp = g_strdup_printf(_("Spam revoked: %d messages"), spam?spam:0);
	label1 = GTK_WIDGET (gtk_builder_get_object(stuff->xml, "label23"));
	gtk_label_set_text_with_mnemonic(GTK_LABEL(label1), tmp);
	g_free(tmp);

	page = gtk_vbox_new (FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(page), 12);
	tab_label = gtk_label_new (_("Report Spam"));
	gtk_notebook_append_page (
		GTK_NOTEBOOK (hook_data->parent), page, tab_label);

	gtk_box_pack_start (
		GTK_BOX (page),
		GTK_WIDGET (gtk_builder_get_object(stuff->xml, "settingsbox")),
		FALSE,
		FALSE,
		0);

	/* Clean up */
//	g_signal_connect (page, "destroy", G_CALLBACK (cleanup_cb), stuff);

	return page;
}

CamelStore *
rspam_component_peek_local_store(void)
{
#if (EVOLUTION_VERSION < 22900) //kb//
	return mail_component_peek_local_store(NULL);
#else
#if (EVOLUTION_VERSION < 30303)
	return e_mail_local_get_store();
#else
	EMailBackend *backend;
	EMailSession *session;
	EShellBackend *shell_backend;

	shell_backend = e_shell_view_get_shell_backend (main_shell_view);

	backend = E_MAIL_BACKEND (shell_backend);
	session = e_mail_backend_get_session (backend);
	return e_mail_session_get_local_store (session);
#endif
#endif
}

gchar *
rspam_lookup_uri_by_folder_name(gchar *name)
{
	CamelFolder *folder;
	gchar *uri;
	CamelStore *store = rspam_component_peek_local_store();

	if (!name)
		return NULL;

#if (DATASERVER_VERSION >= 2033001)
	folder = camel_store_get_folder_sync (store, name, 0, NULL, NULL);
#else
	folder = camel_store_get_folder (store, name, 0, NULL);
#endif
	if (!folder) return NULL;
#if EVOLUTION_VERSION >= 30101
	uri = e_mail_folder_uri_from_folder (folder);
#else
#if EVOLUTION_VERSION >= 29101
	uri = (gchar *)camel_folder_get_uri (folder);
#else
	uri = mail_tools_folder_to_url (folder);
#endif
#endif
	return uri;
}

void
rspam_folder_cb (GtkWidget *widget, gpointer data)
{
#if EVOLUTION_VERSION >= 30101
	EMailBackend *backend;
	EMailSession *session;
	const gchar *folderinfo;
#else
	CamelFolderInfo *folderinfo;
#endif
	GtkWidget *dialog;
	GtkWindow *window;
	const gchar *uri;
	struct _copy_folder_data *cfd;
	EMFolderSelector *selector;
#if EVOLUTION_VERSION >= 30101
	GError *error = NULL;
	EMFolderTree *folder_tree;
#if EVOLUTION_VERSION >= 30101
	EMFolderTreeModel *model;
#endif
#else
	GtkWidget *folder_tree;
#endif
	gchar *folder_name = NULL;
	EMailReader *reader;
	EShellContent *shell_content;

	gchar *text = (gchar *)gtk_label_get_text(GTK_LABEL(data));

	shell_content = e_shell_view_get_shell_content (main_shell_view);
	reader = E_MAIL_READER (shell_content);

#if EVOLUTION_VERSION >= 30101
	backend = e_mail_reader_get_backend (reader);
	session = e_mail_backend_get_session (backend);
#endif

	window = e_mail_reader_get_window (reader);


#if EVOLUTION_VERSION >= 30103
	model = em_folder_tree_model_get_default ();
#endif
#if EVOLUTION_VERSION >= 30303
	dialog = em_folder_selector_new (
			window,
			model
#if EVOLUTION_VERSION < 31301
			, EM_FOLDER_SELECTOR_CAN_CREATE,
			_("Move to Folder"), NULL, _("M_ove"));
#else
		);
	em_folder_selector_set_can_create (EM_FOLDER_SELECTOR (dialog), TRUE);
	em_folder_selector_set_caption (EM_FOLDER_SELECTOR (dialog), _("Move to Folder"));
	em_folder_selector_set_default_button_label (EM_FOLDER_SELECTOR (dialog), _("M_ove"));
#endif /* 31190 */

#else
#if EVOLUTION_VERSION < 30101
	folder_tree = em_folder_tree_new ();
#endif
	dialog = em_folder_selector_new (
			window,
#if EVOLUTION_VERSION >= 30101
			backend,
#if EVOLUTION_VERSION >= 30103
			model,
#endif
#else
			EM_FOLDER_TREE (folder_tree),
#endif
			EM_FOLDER_SELECTOR_CAN_CREATE,
			_("Move to Folder"), NULL, _("M_ove"));
#endif

	selector = EM_FOLDER_SELECTOR (dialog);
#if EVOLUTION_VERSION >= 30101
	folder_tree = em_folder_selector_get_folder_tree (selector);
#endif
	emu_restore_folder_tree_state (EM_FOLDER_TREE (folder_tree));

	em_folder_tree_set_excluded (
#if EVOLUTION_VERSION >= 30101
		folder_tree,
		EMFT_EXCLUDE_NOSELECT |
		EMFT_EXCLUDE_VIRTUAL |
#else
		EM_FOLDER_TREE (folder_tree),
		EMFT_EXCLUDE_NOSELECT | EMFT_EXCLUDE_VIRTUAL |
#endif
		EMFT_EXCLUDE_VTRASH);

	if ((uri = rspam_lookup_uri_by_folder_name(text)))
#if EVOLUTION_VERSION >= 30101
		em_folder_tree_set_selected (
			folder_tree, uri, FALSE);
#else
		em_folder_selector_set_selected (
			EM_FOLDER_SELECTOR (dialog),
			uri);
#endif

#if EVOLUTION_VERSION >= 30101
	folderinfo = em_folder_tree_get_selected_uri ((EMFolderTree *)folder_tree);
#else
	folderinfo = em_folder_tree_get_selected_folder_info ((EMFolderTree *)folder_tree);
#endif

	cfd = g_malloc (sizeof (*cfd));
#if EVOLUTION_VERSION < 30101
	cfd->fi = folderinfo;
#endif
	cfd->delete = 1;

#if EVOLUTION_VERSION >= 30101
	e_mail_folder_uri_parse (
		CAMEL_SESSION (session), folderinfo,
		&cfd->source_store, &cfd->source_folder_name, &error);

	if (error != NULL) {
		g_warning ("%s", error->message);
		g_error_free (error);
		g_free (cfd);
		return;
	}
#endif

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK) {
#if EVOLUTION_VERSION >= 30101
		EMailBackend *backend;
		EMailReader *reader;
		EShellContent *shell_content;
		CamelStore *store;
		EMailSession *session;
#endif
		uri = em_folder_selector_get_selected_uri (
			EM_FOLDER_SELECTOR (dialog));

#if EVOLUTION_VERSION >= 30101
		shell_content = e_shell_view_get_shell_content (main_shell_view);
		reader = E_MAIL_READER (shell_content);
		backend = e_mail_reader_get_backend (reader);
		session = e_mail_backend_get_session (backend);
		e_mail_folder_uri_parse (
			CAMEL_SESSION(session), uri,
			&store, &folder_name, NULL);
#else
		folder_name = em_utils_folder_name_from_uri(uri);
#endif
		gtk_label_set_text(GTK_LABEL(data), folder_name);
		g_free(folder_name);
#if EVOLUTION_VERSION < 30304
		gconf_client_set_string (
			rspam_gconf,
			GCONF_KEY_MOVE_FOLDER, uri, NULL);
#else
		g_settings_set_string (
			rspam_settings,
			CONF_KEY_MOVE_FOLDER, uri);
#endif
	}

	gtk_widget_destroy (dialog);
}

#if EVOLUTION_VERSION >= 31190
#define ESEventTargetUpgrade void
#endif

#if EVOLUTION_VERSION < 22900
void org_gnome_sa_rspam_startup(void *ep, EMPopupTargetSelect *t);
#else
void org_gnome_sa_rspam_startup(void *ep, ESEventTargetUpgrade *t);
#endif

#if EVOLUTION_VERSION < 22900
void org_gnome_sa_rspam_startup(void *ep, EMPopupTargetSelect *t)
#else
void org_gnome_sa_rspam_startup(void *ep, ESEventTargetUpgrade *t)
#endif
{
}

gboolean
check_discovery(void)
{
	gchar *stfile = g_build_filename(
				(gchar *)e_get_user_data_dir(),
				".rspam.disc", NULL);
	struct timeval start;
	FILE *f = NULL;
	gboolean ret = TRUE;
	unsigned long int remain;
	gchar rfeed[80];
	memset(rfeed, 0, 79);
	gettimeofday(&start, NULL);
	if (!g_file_test(stfile, G_FILE_TEST_EXISTS)) {
		if ((f = fopen(stfile, "w"))) {
			fprintf(f, "%lu", start.tv_sec);
			fclose(f);
		}
		ret = TRUE;
		goto out;
	}
	if ((f = fopen(stfile, "r+"))) {
		fgets(rfeed, 50, f);
		remain = start.tv_sec - strtoul(
			(const char *)&rfeed, NULL, 10);
		if (DISCOVERY_TTL <= remain) {
			(void)fseek(f, 0L, SEEK_SET);
			fprintf(f, "%lu", start.tv_sec);
			fclose(f);
			ret =  TRUE;
			goto out;
		} else {
			fclose(f);
			ret = FALSE;
		}
	}
out:    g_free(stfile);
	return ret;
}

#if EVOLUTION_VERSION >= 29101
EMailSession*
_get_mail_session(void);

EMailSession*
_get_mail_session(void)
{
	EMailBackend *backend;
	EMailReader *reader;
	EShellContent *shell_content;
	shell_content = e_shell_view_get_shell_content (main_shell_view);
	reader = E_MAIL_READER (shell_content);
	backend = e_mail_reader_get_backend (reader);
	return e_mail_backend_get_session (backend);
}
#endif

void
#if (DATASERVER_VERSION >= 2031002)
rs_online(EShell *shell)
#else
rs_online(CamelObject *o, void *event_data, void *data)
#endif
{
#if (DATASERVER_VERSION >= 2031002)
	rspam_online =  e_shell_get_online (shell);
#else
	rspam_online =  camel_session_is_online ((CamelSession *)o);
#endif
	if (rspam_online && check_discovery()) {
		while (gtk_events_pending())
			gtk_main_iteration ();
		taskbar_push_message(_("Rspam: Performing discovery (..)"));
		/*discover pyzor*/
#if EVOLUTION_VERSION < 30304
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_ENABLE_PYZOR, NULL))
#else
		if (g_settings_get_boolean (rspam_settings, CONF_KEY_ENABLE_PYZOR))
#endif
			pyzor_status = pyzor_discover_cb(NULL, NULL);
		/*discover razor*/
#if EVOLUTION_VERSION < 30304
		if (gconf_client_get_bool (rspam_gconf, GCONF_KEY_ENABLE_RAZOR, NULL))
#else
		if (g_settings_get_boolean (rspam_settings, CONF_KEY_ENABLE_RAZOR))
#endif
			discover_cb(NULL, stuff);
		taskbar_pop_message();
	while (gtk_events_pending())
		gtk_main_iteration ();
	}
}

static GtkActionEntry entries[] = {
	{ "report-as-spam",
	"stock_spam",
	N_("_Report as SPAM"),
	NULL,
	N_("Report current message to various spam filtering networks"),
	G_CALLBACK (org_gnome_sa_rspam) },

	{ "revoke-spam",
	"mail-mark-notjunk",
	N_("Revo_ke SPAM"),
	NULL,
	N_("Whithdraws reported message as being spam"),
	G_CALLBACK (org_gnome_sa_revoke) },
};

#if EVOLUTION_VERSION >= 22900 //kb//
static EPopupActionEntry popup_entries[] = {
	{ "report-as-spam-popup",
	NULL,
	"report-as-spam" },

	{ "revoke-spam-popup",
	NULL,
	"revoke-spam" },
};

gboolean
e_plugin_ui_init (GtkUIManager *ui_manager,
		EShellView *shell_view);

gboolean
e_plugin_ui_init (GtkUIManager *ui_manager,
		EShellView *shell_view)
{
	EShellWindow *shell_window;
	GtkActionGroup *action_group;

	shell_window = e_shell_view_get_shell_window (shell_view);
	action_group = e_shell_window_get_action_group (
				shell_window,
				"mail");
	main_shell_view = shell_view;


#if EVOLUTION_VERSION >= 30101
	shell_backend = e_shell_view_get_shell_backend (shell_view);
	backend = E_MAIL_BACKEND (shell_backend);
	session = e_mail_backend_get_session (backend);
#endif

#if (DATASERVER_VERSION >= 2031002)
	g_signal_connect(
#if EVOLUTION_VERSION < 22900
		mail_component_peek_session(NULL),
#else
		e_shell_get_default(),
#endif
		"notify::online",
		G_CALLBACK(rs_online),
		NULL);
	/*shell doesn't trigger online event ?*/
	rs_online(e_shell_get_default());
#else
	camel_object_hook_event(
#if EVOLUTION_VERSION < 22900
		mail_component_peek_session(NULL),
#else
		session,
#endif
		"online",
		(CamelObjectEventHookFunc)rs_online,
		NULL);
#endif

	gtk_action_group_add_actions (
		action_group,
		entries,
		G_N_ELEMENTS (entries),
		shell_view);
	e_action_group_add_popup_actions (
		action_group,
		popup_entries,
		G_N_ELEMENTS (popup_entries));

//        g_signal_connect (
//              shell_view, "update-actions",
//            G_CALLBACK (update_actions_cb), NULL);

	return TRUE;
}
#endif

void
rspam_finalize(void)
{
	g_print("RSPAM: cleaning temp files.\n");
	g_list_foreach(tlist, (GFunc)unlink, NULL);
	g_list_foreach(tlist, (GFunc)g_free, NULL);
}

#if EVOLUTION_VERSION < 22900
int e_plugin_lib_enable(EPluginLib *ep, int enable);
#else
int e_plugin_lib_enable(EPlugin *ep, int enable);
#endif

int
#if EVOLUTION_VERSION < 22900
e_plugin_lib_enable(EPluginLib *ep, int enable)
#else
e_plugin_lib_enable(EPlugin *ep, int enable)
#endif
{
	if (enable) {
		bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
		bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
		printf("Rspam Plugin enabled (evolution %s, evolution-rspam %s)\n",
			EVOLUTION_VERSION_STRING,
			VERSION);
#if EVOLUTION_VERSION < 30304
		rspam_gconf = gconf_client_get_default();
#else
		rspam_settings = g_settings_new(CONF_SCHEMA);
#endif
		/* A structure to pass some stuff around */
		stuff = g_new0 (struct rspam_stuff, 100);

		readrazorconfig(stuff);
		atexit(rspam_finalize);
	} else {
		printf("Rspam Plugin disabled\n");
	}

	return 0;
}
